using System;
using System.IO;
using System.Threading;
using SQLReaderPerformance.SqlDataReaderTest;
using ObjectPoolingPerformance;

namespace SQLReaderPerformance
{
	/// <summary>
	/// Class1   Դϴ.
	/// </summary>
	class MainClass
	{
		private static System.Random random = new Random(DateTime.Now.Millisecond);
		/// <summary>
		/// ش  α׷  Դϴ.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// string usingColumnNameLogPath = "UsingColumnName.txt";
			// UsingColumnNamePerformanceTest(usingColumnNameLogPath);

			// string castingLogPath = "casting.txt";
			// CastingPerformanceTest(castingLogPath);

			// Object Pooling? Connectino Pooling?
			/*
			int max = 20;
			Thread[] threads = new Thread[max];

			for(int i=0; i<max; i++)
			{
				threads[i] = new Thread(new ThreadStart(MainClass.PoolingPerformanceTest));
			}

			for(int i=0; i<max; i++)
			{
				threads[i].Start();
			}


			for(int i=0; i<max; i++)
			{
				threads[i].Join();
			}
			*/
		}

		private static void PoolingPerformanceTest()
		{
			string poolingLogPath = random.Next().ToString() + ".txt";

			using(StreamWriter sw = new StreamWriter(poolingLogPath,false))
			{
				int countPoolingIsFaster = 0;
				int countNonPoolingIsFaster = 0;
				double countTicks = 0;

				for(int index=0; index<1000; index++)
				{
					Pooling pooling = new Pooling();
					NonPooling nonPooling = new NonPooling();

					pooling.Retrieve();
					nonPooling.Retreive();

					long prevPooling = 0;
					long prevNonPooling = 0;
			
					long intervalPooling = 0;
					long intervalNonPooling = 0;

					double totalDifference = 0;
				
					if(index%2 == 0)
					{
						prevNonPooling = DateTime.Now.Ticks;
						nonPooling.Retreive();
						intervalNonPooling = (DateTime.Now.Ticks - prevNonPooling);

						prevPooling = DateTime.Now.Ticks;
						pooling.Retrieve();
						intervalPooling = (DateTime.Now.Ticks - prevPooling);
					}
					else
					{
						prevPooling = DateTime.Now.Ticks;
						pooling.Retrieve();
						intervalPooling = (DateTime.Now.Ticks - prevPooling);

						prevNonPooling = DateTime.Now.Ticks;
						nonPooling.Retreive();
						intervalNonPooling = (DateTime.Now.Ticks - prevNonPooling);
					}

					totalDifference += intervalPooling - intervalNonPooling;

					if( totalDifference > 0 ) 
					{
						Console.WriteLine(String.Format("NonPooling is faster : {0}",totalDifference));
						countNonPoolingIsFaster++;
					}
					else if(totalDifference == 0)
					{
						Console.WriteLine(String.Format("Equal"));
					}
					else
					{
						Console.WriteLine(String.Format("Pooling is faster : {0}",totalDifference));
						countPoolingIsFaster++;
					}

					countTicks += totalDifference;
					sw.WriteLine(totalDifference);
					sw.Flush();
				}

				Console.WriteLine(String.Format("countPoolingIsFaster : {0}",countPoolingIsFaster));
				Console.WriteLine(String.Format("countNonPoolingIsFaster : {0}",countNonPoolingIsFaster));

				Console.WriteLine(String.Format("countTicks : {0}",countTicks));
			}
		}

		private static void CastingPerformanceTest(string castingLogPath)
		{
			using(StreamWriter sw = new StreamWriter(castingLogPath,false))
			{
				int countCastingIsFaster = 0;
				int countGetDataTypeIsFaster = 0;
				double countTicks = 0;

				for(int index=0; index<1000; index++)
				{
					Casting casting = new Casting();

					casting.Cast();
					casting.GetDataType();

					long prevCasting = 0;
					long prevGetDatType = 0;
			
					long intervalCasting = 0;
					long intervalGetDataType = 0;

					double totalDifference = 0;
				
					if(index%2 == 0)
					{
						prevGetDatType = DateTime.Now.Ticks;
						casting.GetDataType();
						intervalGetDataType = (DateTime.Now.Ticks - prevGetDatType);

						prevCasting = DateTime.Now.Ticks;
						casting.Cast();
						intervalCasting = (DateTime.Now.Ticks - prevCasting);
					}
					else
					{
						prevCasting = DateTime.Now.Ticks;
						casting.Cast();
						intervalCasting = (DateTime.Now.Ticks - prevCasting);

						prevGetDatType = DateTime.Now.Ticks;
						casting.GetDataType();
						intervalGetDataType = (DateTime.Now.Ticks - prevGetDatType);;
					}

					totalDifference += intervalCasting - intervalGetDataType;

					if( totalDifference > 0 ) 
					{
						Console.WriteLine(String.Format("GetDataType is faster : {0}",totalDifference));
						countGetDataTypeIsFaster++;
					}
					else if(totalDifference == 0)
					{
						Console.WriteLine(String.Format("Equal"));
					}
					else
					{
						Console.WriteLine(String.Format("Casting is faster : {0}",totalDifference));
						countCastingIsFaster++;
					}

					countTicks += totalDifference;
					sw.WriteLine(totalDifference);
					sw.Flush();
				}

				Console.WriteLine(String.Format("countCastingIsFaster : {0}",countCastingIsFaster));
				Console.WriteLine(String.Format("countGetDataTypeIsFaster : {0}",countGetDataTypeIsFaster));

				Console.WriteLine(String.Format("countTicks : {0}",countTicks));
			}
		}

		private static void UsingColumnNamePerformanceTest(string usingColumnNameLogPath)
		{
			using(StreamWriter sw = new StreamWriter(usingColumnNameLogPath,false))
			{
				int countUsingIsFaster = 0;
				int countNotUsingIsFaster = 0;
				double countTicks = 0;

				for(int index=0; index<1000; index++)
				{
					SqlDataReaderTest.UsingColumnName usingColumnName = new UsingColumnName();

					usingColumnName.Using();
					usingColumnName.NotUsing();

					long prevUsing = 0;
					long prevNotUsing = 0;
			
					long intervalUsing = 0;
					long intervalNotUsing = 0;

					double totalDifference = 0;
				
					if(index%2 == 0)
					{
						prevNotUsing = DateTime.Now.Ticks;
						usingColumnName.NotUsing();
						intervalNotUsing = (DateTime.Now.Ticks - prevNotUsing);

						prevUsing = DateTime.Now.Ticks;
						usingColumnName.Using();
						intervalUsing = (DateTime.Now.Ticks - prevUsing);

						totalDifference += intervalUsing - intervalNotUsing;
					}
					else
					{
						prevUsing = DateTime.Now.Ticks;
						usingColumnName.Using();
						intervalUsing = (DateTime.Now.Ticks - prevUsing);

						prevNotUsing = DateTime.Now.Ticks;
						usingColumnName.NotUsing();
						intervalNotUsing = (DateTime.Now.Ticks - prevNotUsing);

						totalDifference += intervalUsing - intervalNotUsing;
					}

					if( totalDifference > 0 ) 
					{
						Console.WriteLine(String.Format("NotUsing is faster : {0}",totalDifference));
						countNotUsingIsFaster++;
					}
					else if(totalDifference == 0)
					{
						Console.WriteLine(String.Format("Equal"));
					}
					else
					{
						Console.WriteLine(String.Format("Using is faster : {0}",totalDifference));
						countUsingIsFaster++;
					}

					countTicks += totalDifference;
					sw.WriteLine(totalDifference);
				}

				Console.WriteLine(String.Format("countUsingIsFaster : {0}",countUsingIsFaster));
				Console.WriteLine(String.Format("countNotUsingIsFaster : {0}",countNotUsingIsFaster));

				Console.WriteLine(String.Format("countTicks : {0}",countTicks));
			}
		}
	}
}
